from testtools import TestCase
from testtools.matchers import Contains
from testtools.content import text_content
import unittest
import zmq

from utils import run_command, update_pxe_config, CLUSTER_CONTROLLER_IP



tests_order = [
    'test_preseed_updated_cluster_config',
    'test_update_pxe_config',
    'test_import_pxe_files',
    'test_wait_for_region_controller',
]


def sorting_method(ignored, first_test, second_test):
    return tests_order.index(first_test) - tests_order.index(second_test)


unittest.TestLoader.sortTestMethodsUsing = sorting_method


class ClusterControllerIntegration(TestCase):

    def test_preseed_updated_cluster_config(self):
        # Make sure the cluster config was updated by the seed file.
        maas_fd = open("/etc/maas/maas_cluster.conf" , "r+")
        maas_file = maas_fd.read()
        self.assertThat(maas_file, Contains(
            'MAAS_URL=http://192.168.21.5/MAAS'))

    def test_update_pxe_config(self):
        update_pxe_config()

    def test_import_pxe_files(self):
        output, err = run_command(["maas-import-pxe-files"])
        self.assertThat(output, Contains('Downloading to temporary location'))
        # XXX: matsubara Bug=1076444
        # maas-import-pxe-files outputs to stderr during normal operation.
        #self.assertIs(err, '')

    def test_wait_for_region_controller(self):
        """Wait for the region controller to run the integration tests.

        The region controller will send a message to the cluster controller
        once all tests finish.
        """
        context = zmq.Context()
        socket = context.socket(zmq.REP)
        socket.bind('tcp://%s:5555' % CLUSTER_CONTROLLER_IP)
        msg = ''
        while msg != "Region controller tests finished.":
            msg = socket.recv()
            self.addDetail(
                "Waiting for region controller to signal end of tests",
                text_content(
                    "Waiting signal from region controller: '%s'" % msg))

    @classmethod
    def tearDownClass(cls):
        """Power off the cluster controller VM after the test run."""
        run_command(["sudo", "poweroff"])
